# coffeelint: disable=max_line_length
Eventemitter = require('events').EventEmitter
emitter = new Eventemitter()
child_process = require 'child_process'
_ = require 'lodash'
logger = require("../../../shared_modules/logger")
helpers = require './channels-helpers'
iconvlite = require("iconv-lite")
fs = require 'fs'
async = require 'async'
ko = require 'knockout'

running = true

bchannelsList = []#.extend({rateLimit: 10, notify: "always"})
getBchannelList = ->
  logger.info "Updating list of B-Channels"
  child_process.exec "getchannels -m -b 2>&1 | grep -v 'Channels found'", (error, stdout) -> # omitted stderr
    stdout = stdout.toString()
    channels = stdout.split("\n")
    channels = _.filter channels, (line) ->
      (line isnt "") and (line.indexOf("B-Channels and") is -1)
    # _.each channels, (bchannelName) ->
    #   bchannelsLocal[bchannelName] = {fullName: bchannelName}
    bchannelsList = channels
    logger.silly {bchannelsList}
    logger.info "List of B-Channels updated"

    # touch all files in /tmp to trigger update of currently running calls
    child_process.exec "touch /tmp/*"
    emitter.emit "changed"
getBchannelList()

bchannelConfigs = {}
watchDir = "/tmp/"

readChangedFiles = async.queue (filename, cb) ->

  logger.info "Reading #{filename}"
  fs.readFile "#{watchDir}#{filename}", (err, content) ->
    if err
      if err.code is "ENOENT"
        logger.warn "#{filename} does not exist"
      else
        logger.error "Could not read #{filename}"
        logger.error err
      return cb()
    decodedContent = iconvlite.decode(content, "utf-8")
    channelProperties = decodedContent.split("--")
    completeBchannelInfo = {}
    logger.debug "Updating channel info with infos from file"
    _.each channelProperties, (keyValObject) ->
      tempArray = keyValObject.split("=")
      newKey = tempArray[0]
      newVal = tempArray[1]
      try
        newKey = JSON.parse(newKey)
      try
        newVal = JSON.parse(newVal)
      completeBchannelInfo[newKey] = newVal

    bchannelName = completeBchannelInfo.bchannel

    if completeBchannelInfo.error is "none" or completeBchannelInfo.error is "active"
      logger.info "#{bchannelName} is in use (#{completeBchannelInfo.error})"
      completeBchannelInfo.active = true
    else
      logger.info "#{bchannelName} is idle"
      completeBchannelInfo.active = false

    bchannelConfigs[bchannelName] = completeBchannelInfo
    emitter.emit "changed"
    cb()
, 1

changedFiles = ko.observableArray().extend { rateLimit: 1000 }
changedFiles.subscribe (files) ->
  return unless running
  files = _.uniq files
  if files.length
    logger.debug "Changed files in #{watchDir}: #{files.length}"
    changedFiles.removeAll()
  for oneFile in files
    readIfBchannel oneFile

readIfBchannel = (filename) ->
  if (filename.lastIndexOf("BChannelState", 0) is 0) #and (changedFiles.indexOf(filename) is -1)
    logger.info "#{filename} changed"
    # debug "#{filename}, #{event}"
    readChangedFiles.push filename

#start watching /tmp and scan changed files for changes
logger.info "Watching #{watchDir} for changes"
watcher = fs.watch watchDir, (event, filename) ->
  if running
    changedFiles.push filename
    # readIfBchannel filename

readAllBchannels = ->
  logger.info("Getting infos about all B-Channels")
  fs.readdir watchDir, (err, files) ->
    readIfBchannel oneFile for oneFile in files
    unless files.length
      emitter.emit "changed"


activeCallsInterval = undefined

emitter.on "changed", _.throttle ->
  logger.debug "At least one channel status changed"
  localBchannels = []
  _.each bchannelsList, (bchan) ->
    bchanconfig = _.find bchannelConfigs, (conf) ->
      conf.bchannel is bchan

    if bchanconfig
      bchanconfig.runTime = helpers.calculateRuntime(bchanconfig.date)
    else
      bchanconfig = {
        bchannel: bchan
      }

    localBchannels.push bchanconfig


  # get active calls
  activeCalls = _.filter bchannelConfigs, (conf) ->
    conf.active

  logger.info "Active calls: #{activeCalls.length}"

  # reset interval to send active calls runTime
  clearInterval activeCallsInterval if activeCallsInterval

  # if there are active calls, start a new interval for their runTime
  if _.size(activeCalls)
    activeCallsInterval = setInterval ->
      unless running
        clearInterval activeCallsInterval if activeCallsInterval
      logger.debug "Recalculating runtime of active calls"
      _.each activeCalls, (conf) ->
        conf.runTime = helpers.calculateRuntime(conf.date)
      emitter.emit "new", localBchannels
    , 2000

  emitter.emit "new", localBchannels

, 2000, { leading: true }

bchannelsLib = emitter

bchannelsLib.start = ->
  if running
    logger.debug "Already running"
    return
  readAllBchannels()
  running = true
  logger.info "Starting tracking of B-Channels"

bchannelsLib.stop = ->
  unless running
    logger.debug "Already stopped"
    return
  running = false
  logger.info "Stopping tracking of B-Channels"

module.exports = bchannelsLib
